@@ -5,7 +5,7 @@ use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '0.21';
+our $VERSION = '0.22';
 
 use Statistics::Covid::Utils;
 use Statistics::Covid::Datum;
@@ -56,29 +56,10 @@ sub	new {
 	$self->config($config_hash);
 
 	if( exists($params->{'providers'}) && defined($m=$params->{'providers'}) ){
-		my %providers = ();
-		for my $aprovider (@$m){
-			my $pn = 'Statistics::Covid::DataProvider::'.$aprovider;
-			my $pnf = File::Spec->catdir(split(/\:\:/, $pn)).'.pm';
-			my $loadedOK = eval {
-				require $pnf;
-				$pn->import;
-				1;
-			};
-			if( ! $loadedOK ){ warn "error, failed to load module '$pn' (file '$pnf'), most likely provider does not exist '$pn'."; return undef }
-			my $providerObj = $pn->new({
-				'config-hash' => Storable::dclone($config_hash),
-				'debug' => $debug
-			});
-			if( ! defined $providerObj ){ warn "error, call to $pn->new() has failed."; return undef }
-			# the key to the providers can be the full package or just this bit
-			# we prefer this bit (e.g. World::JHU)
-			#$providers{$pn} = $providerObj;
-			$providers{$aprovider} = $providerObj;
-			if( $debug > 0 ){ warn "provider added '$pn':\n".$providerObj->toString() }
-		}
-		$self->{'p'}->{'provider-objs'} = {%providers};
-	} else { warn "error, 'providers' was not specified, a list of provider names, e.g. 'UK::GOVUK' and/or 'World::JHU' must be provided."; return undef }
+		if( ! $self->providers($m) ){ warn "error, failed to install specified provider(s) (calling providers()) : '".join("','", @$m)."'."; return undef }
+	} else {
+		if( $debug > 0 ){ warn "warning, 'providers' (like 'UK::GOVUK' and/or 'World::JHU') was not specified, that's ok, but you must insert some before fetching any data - interacting with db will be ok." }
+	}
 
 	for(keys %$params){
 		$self->{$_} = $params->{$_} if exists $self->{$_}
@@ -108,6 +89,7 @@ sub	fetch_and_store {
 	my $debug = $self->debug();
 	my $num_fetched_total = 0;
 	my $providers = $self->providers();
+	if( ! defined $providers ){ warn "error, data providers must be inserted prior to using this, e.g. providers('World::JHU')."; return undef }
 	for my $pn (keys %$providers){
 		my $providerObj = $providers->{$pn};
 		my $datas = $providerObj->fetch();
@@ -163,22 +145,27 @@ sub	select_datums_from_db {
 	return $objs
 }
 # shortcut to selecting datum objects from db (select_datums_from_db())
-# from a single location
+# from a single location and ordering the results in time-ascending order.
 # it's useful for getting the timeline for a given place
 # if successful, it returns an array of Datum objects (sorted on time)
 # it returns undef on failure
-sub	select_datums_from_db_for_location {
+sub	select_datums_from_db_for_specific_location_time_ascending {
 	my $self = $_[0];
-	my $location_name = $_[1]; # exactly as it is stored in DB
-	my $belongsto = $_[2]; #optionally specify a 'belongsto' (e.g. UK)
+	# this can be an exact location name (case sensitive)
+	# OR it can be this {'like' => 'Ha%'}
+	# which does a wildcard search
+	my $location_condition = $_[1];
+	#optionally specify a 'belongsto' (e.g. UK)
+	# either exact or wildcard, like above
+	my $belongsto_condition = $_[2];
 
-	my $conditions = {'name'=>$location_name};
-	$conditions->{'belongsto'} = $belongsto if defined $belongsto;
+	my $conditions = {'name'=>$location_condition};
+	$conditions->{'belongsto'} = $belongsto_condition if defined $belongsto_condition;
 
 	my $results = $self->select_datums_from_db({
 		'conditions' => $conditions,
 		'attributes' => {
-			'order_by' => 'datetimeUnixEpoch'
+			'order_by' => {'-asc' => 'datetimeUnixEpoch'}
 		},
 	});
 	if( ! defined $results ){ warn "error, call to ".'select_datums_from_db()'." has failed."; return undef }
@@ -257,7 +244,9 @@ sub	read_data_from_files {
 	my @providerstrs;
 	if( ! exists($params->{'what'}) || ! defined($params->{'what'}) ){
 		# nothing was provided in the input, we use ALL our providers loaded during construction
-		@providerstrs = keys %{$self->providers()};
+		my $m = $self->providers();
+		if( ! defined $m ){ warn "error, data providers must be inserted prior to using this, e.g. providers('World::JHU')."; return undef }
+		@providerstrs = keys %$m;
 	} else {
 		# something was given at input
 		$inp = $params->{'what'};
@@ -382,12 +371,44 @@ sub     dbparams { return $_[0]->config()->{'dbparams'} }
 # if a provider id is specified at input)
 sub	providers {
 	my $self = $_[0];
-	my $pstr = $_[1];
-	if( defined $pstr ){
+	my $m = $_[1];
+
+	if( ! defined $m ){ return $self->{'p'}->{'provider-objs'} }
+
+	if( ref($m) eq '' ){
+		# we were given a string to search for that provider and return its data
 		# return the exact provider if the pstr matches an id from our providers
-		return($self->{'p'}->{'provider-objs'}->{$pstr})
-			if exists($self->{'p'}->{'provider-objs'}->{$pstr});
+		return($self->{'p'}->{'provider-objs'}->{$m})
+			if exists($self->{'p'}->{'provider-objs'}->{$m});
 		return undef # id given is not in our list
+	} elsif( ref($m) eq 'ARRAY' ){
+		my $debug = $self->debug();
+		# we were given an arrayref, presumably a list of providers
+		# we need to find the package of each provider and load it,
+		# that's why the contents of this array plus the package string below
+		# must much exactly our installed provider packages
+		my %providers = ();
+		for my $aprovider (@$m){
+			my $pn = 'Statistics::Covid::DataProvider::'.$aprovider;
+			my $pnf = File::Spec->catdir(split(/\:\:/, $pn)).'.pm';
+			my $loadedOK = eval {
+				require $pnf;
+				$pn->import;
+				1;
+			};
+			if( ! $loadedOK ){ warn "error, failed to load module '$pn' (file '$pnf'), most likely provider does not exist '$pn'."; return undef }
+			my $providerObj = $pn->new({
+				'config-hash' => Storable::dclone($self->config()),
+				'debug' => $debug
+			});
+			if( ! defined $providerObj ){ warn "error, call to $pn->new() has failed."; return undef }
+			# the key to the providers can be the full package or just this bit
+			# we prefer this bit (e.g. World::JHU)
+			#$providers{$pn} = $providerObj;
+			$providers{$aprovider} = $providerObj;
+			if( $debug > 0 ){ warn "provider added '$pn':\n".$providerObj->toString() }
+		}
+		$self->{'p'}->{'provider-objs'} = \%providers;
 	}
 	return $self->{'p'}->{'provider-objs'} # that's the hash with the providers
 }
@@ -464,7 +485,7 @@ Statistics::Covid - Fetch, store in DB, retrieve and analyse Covid-19 statistics
 
 =head1 VERSION
 
-Version 0.21
+Version 0.22
 
 =head1 DESCRIPTION
 
@@ -529,7 +550,11 @@ solidify.
 	for (@$someObjs);
 	
 	# or for a single place (this sub sorts results wrt publication time)
-	my $timelineObjs = $covid->select_datums_from_db_for_location('Hackney');
+	my $timelineObjs = $covid->select_datums_from_db_for_specific_location_time_ascending('Hackney');
+	# or for a wildcard match
+	# $covid->select_datums_from_db_for_specific_location_time_ascending({'like'=>'Hack%'});
+	# and maybe specifying max rows
+	# $covid->select_datums_from_db_for_specific_location_time_ascending({'like'=>'Hack%'}, {'rows'=>10});
 	for my $anobj (@$timelineObjs){
 		print $anobj->toString()."\n";
 	}
